{% extends "base.html" %}

{% block outdated %}
  This page is for an outdated or development version.
  <a href="{{ '../' ~ base_url }}">
    <strong>Click here to go to the latest stable version.</strong>
  </a>
{% endblock %}
